@@ -70,6 +70,20 @@ static struct {
 	            sizeof(((send_ctx_t *)0)->rdata)];
 } ctl_globals;
 
+static bool allow_blocking_while_ctl_txn(zone_event_type_t event)
+{
+	// this can be allowed for those events that do NOT create a zone_update_t
+	switch (event) {
+	case ZONE_EVENT_UFREEZE:
+	case ZONE_EVENT_UTHAW:
+	case ZONE_EVENT_NOTIFY:
+	case ZONE_EVENT_FLUSH:
+		return true;
+	default:
+		return false;
+	}
+}
+
 /*!
  * Evaluates a filter pair and checks for conflicting filters.
  *
@@ -110,6 +124,10 @@ static int schedule_trigger(zone_t *zone, ctl_args_t *args, zone_event_type_t ev
 	int ret = KNOT_EOK;
 
 	if (ctl_has_flag(args->data[KNOT_CTL_IDX_FLAGS], CTL_FLAG_BLOCKING)) {
+		if (!allow_blocking_while_ctl_txn(event) &&
+		    zone->control_update != NULL) {
+			return KNOT_TXN_EEXISTS;
+		}
 		ret = zone_events_schedule_blocking(zone, event, user);
 	} else if (user) {
 		zone_events_schedule_user(zone, event);
@@ -562,6 +580,10 @@ static int init_backup(ctl_args_t *args, bool restore_mode)
 	// Evaluate filters (and possibly fail) before writing to the filesystem.
 	knot_backup_params_t filters = 0;
 	knot_backup_params_t dflts = restore_mode ? BACKUP_PARAM_DFLT_R : BACKUP_PARAM_DFLT_B;
+
+	// Filter '+keysonly' silently changes all defaults to '+no...'.
+	dflts = MATCH_AND_FILTER(args, BACKUP_PARAM_KEYSONLY) ? BACKUP_PARAM_EMPTY : dflts;
+
 	for (const backup_filter_list_t *item = backup_filters; item->name != NULL; item++) {
 		if (!eval_backup_filters(args, &filters, item, dflts)) {
 			return KNOT_EXPARAM;
@@ -629,6 +651,12 @@ static int zone_backup_cmd(zone_t *zone, ctl_args_t *args)
 		return KNOT_EPROGRESS;
 	}
 
+	if (ctx->restore_mode && zone->control_update != NULL) {
+		log_zone_warning(zone->name, "restoring backup not possible due to open control transaction");
+		ctx->failed = true;
+		return KNOT_TXN_EEXISTS;
+	}
+
 	ctx->zone_count++;
 
 	int ret;
@@ -824,6 +852,11 @@ static int zone_txn_begin(zone_t *zone, _unused_ ctl_args_t *args)
 		return KNOT_TXN_EEXISTS;
 	}
 
+	if (zone->backup_ctx != NULL && zone->backup_ctx->restore_mode) {
+		log_zone_warning(zone->name, "zone restore pending, try opening control transaction later");
+		return KNOT_EAGAIN;
+	}
+
 	zone->control_update = malloc(sizeof(zone_update_t));
 	if (zone->control_update == NULL) {
 		return KNOT_ENOMEM;
