@@ -1,4 +1,4 @@
-/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 #include "libknot/libknot.h"
 #include "knot/dnssec/rrset-sign.h"
+#include "knot/dnssec/zone-nsec.h"
 #include "knot/nameserver/internet.h"
 #include "knot/nameserver/nsec_proofs.h"
 #include "knot/nameserver/query_module.h"
@@ -54,7 +55,7 @@ static int wildcard_visit(knotd_qdata_t *qdata, const zone_node_t *node,
 	return KNOT_EOK;
 }
 
-/*! \brief Synthetizes a CNAME RR from a DNAME. */
+/*! \brief Synthesizes a CNAME RR from a DNAME. */
 static int dname_cname_synth(const knot_rrset_t *dname_rr,
                              const knot_dname_t *qname,
                              knot_rrset_t *cname_rrset,
@@ -134,7 +135,7 @@ static int put_answer(knot_pkt_t *pkt, uint16_t type, knotd_qdata_t *qdata)
 		compr_hint = KNOT_COMPR_HINT_QNAME;
 	}
 
-	unsigned put_rr_flags = (qdata->params->flags & KNOTD_QUERY_FLAG_LIMIT_SIZE) ?
+	unsigned put_rr_flags = (qdata->params->proto == KNOTD_QUERY_PROTO_UDP) ?
 	                        KNOT_PF_NULL : KNOT_PF_NOTRUNC;
 	put_rr_flags |= KNOT_PF_ORIGTTL;
 
@@ -193,6 +194,24 @@ static int put_delegation(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 	                            KNOT_COMPR_HINT_NONE, 0);
 }
 
+static int put_nsec3_bitmap(const zone_node_t *for_node, knot_pkt_t *pkt,
+                            knotd_qdata_t *qdata, uint32_t flags)
+{
+	const zone_node_t *node = node_nsec3_get(for_node);
+	if (node == NULL) {
+		return KNOT_EOK;
+	}
+
+	knot_rrset_t nsec3 = node_rrset(node, KNOT_RRTYPE_NSEC3);
+	if (knot_rrset_empty(&nsec3)) {
+		return KNOT_EOK;
+	}
+
+	knot_rrset_t rrsig = node_rrset(node, KNOT_RRTYPE_RRSIG);
+	return process_query_put_rr(pkt, qdata, &nsec3, &rrsig,
+	                            KNOT_COMPR_HINT_NONE, flags);
+}
+
 /*! \brief Put additional records for given RR. */
 static int put_additional(knot_pkt_t *pkt, const knot_rrset_t *rr,
                           knotd_qdata_t *qdata, knot_rrinfo_t *info, int state)
@@ -203,8 +222,8 @@ static int put_additional(knot_pkt_t *pkt, const knot_rrset_t *rr,
 
 	/* Valid types for ADDITIONALS insertion. */
 	/* \note Not resolving CNAMEs as MX/NS name must not be an alias. (RFC2181/10.3) */
-	static const uint16_t ar_type_list[] = { KNOT_RRTYPE_A, KNOT_RRTYPE_AAAA };
-	static const int ar_type_count = 2;
+	static uint16_t ar_type_list[] = { KNOT_RRTYPE_A, KNOT_RRTYPE_AAAA, KNOT_RRTYPE_SVCB };
+	static const int ar_type_count_default = 2;
 
 	int ret = KNOT_EOK;
 
@@ -220,6 +239,11 @@ static int put_additional(knot_pkt_t *pkt, const knot_rrset_t *rr,
 			flags |= KNOT_PF_NOTRUNC;
 		}
 
+		int ar_type_count = ar_type_count_default, ar_present = 0;
+		if (rr->type == KNOT_RRTYPE_SVCB || rr->type == KNOT_RRTYPE_HTTPS) {
+			ar_type_list[ar_type_count++] = rr->type;
+		}
+
 		uint16_t hint = knot_compr_hint(info, KNOT_COMPR_HINT_RDATA +
 		                                glue->ns_pos);
 		const zone_node_t *gluenode = glue_node(glue, qdata->extra->node);
@@ -234,6 +258,28 @@ static int put_additional(knot_pkt_t *pkt, const knot_rrset_t *rr,
 			if (ret != KNOT_EOK) {
 				break;
 			}
+			ar_present++;
+		}
+
+		if ((rr->type == KNOT_RRTYPE_SVCB || rr->type == KNOT_RRTYPE_HTTPS) &&
+		    ar_present < ar_type_count && have_dnssec(qdata)) {
+			// it would be nicer to have this in solve_additional_dnssec, but
+			// it seems infeasible to transfer all the context there
+
+			// adding an NSEC(3) record proving non-existence of some of the
+			// glue with its bitmap
+			if (knot_is_nsec3_enabled(qdata->extra->contents)) {
+				ret = put_nsec3_bitmap(gluenode, pkt, qdata, flags);
+			} else {
+				knot_rrset_t nsec = node_rrset(gluenode, KNOT_RRTYPE_NSEC);
+				if (!knot_rrset_empty(&nsec)) {
+					ret = process_query_put_rr(pkt, qdata, &nsec, &rrsigs,
+					                           KNOT_COMPR_HINT_NONE, flags);
+				}
+			}
+			if (ret != KNOT_EOK) {
+				break;
+			}
 		}
 	}
 
@@ -345,7 +391,7 @@ static int name_found(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 	uint16_t old_rrcount = pkt->rrset_count;
 	int ret = put_answer(pkt, qtype, qdata);
 	if (ret != KNOT_EOK) {
-		if (ret == KNOT_ESPACE && (qdata->params->flags & KNOTD_QUERY_FLAG_LIMIT_SIZE)) {
+		if (ret == KNOT_ESPACE && (qdata->params->proto == KNOTD_QUERY_PROTO_UDP)) {
 			return KNOTD_IN_STATE_TRUNC;
 		} else {
 			return KNOTD_IN_STATE_ERROR;
@@ -548,10 +594,10 @@ static int solve_authority_dnssec(int state, knot_pkt_t *pkt, knotd_qdata_t *qda
 static int solve_additional(int state, knot_pkt_t *pkt, knotd_qdata_t *qdata,
                             void *ctx)
 {
-	int ret = KNOT_EOK;
+	int ret = KNOT_EOK, rrset_count = pkt->rrset_count;
 
 	/* Scan all RRs in ANSWER/AUTHORITY. */
-	for (uint16_t i = 0; i < pkt->rrset_count; ++i) {
+	for (int i = 0; i < rrset_count; ++i) {
 		knot_rrset_t *rr = &pkt->rr[i];
 		knot_rrinfo_t *info = &pkt->rr_info[i];
 
@@ -658,13 +704,12 @@ int internet_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 		return KNOT_STATE_FAIL;
 	}
 
-	/* Check valid zone, transaction security (optional) and contents. */
+	/* Check if valid zone. */
 	NS_NEED_ZONE(qdata, KNOT_RCODE_REFUSED);
 
-	/* No applicable ACL, refuse transaction security. */
+	/* Check if a TSIG is present. */
 	if (knot_pkt_has_tsig(qdata->query)) {
-		/* We have been challenged... */
-		NS_NEED_AUTH(qdata, ACL_ACTION_NONE);
+		NS_NEED_AUTH(qdata, ACL_ACTION_QUERY);
 
 		/* Reserve space for TSIG. */
 		int ret = knot_pkt_reserve(pkt, knot_tsig_wire_size(&qdata->sign.tsig_key));
@@ -673,7 +718,8 @@ int internet_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 		}
 	}
 
-	NS_NEED_ZONE_CONTENTS(qdata); /* Expired */
+	/* Check if the zone is not empty or expired. */
+	NS_NEED_ZONE_CONTENTS(qdata);
 
 	/* Get answer to QNAME. */
 	qdata->name = knot_pkt_qname(qdata->query);
